// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste
// Changes copyright 2004 Kai Blin

/**\file general.h
 * Some generally usefull functions
 *
 *\author Henri Kuuste
 *\author Kai Blin
 */

#ifndef _EF_GENERAL_H_
#define _EF_GENERAL_H_

#include <string>
#include <iostream>

/**
 * Fix the path slashes, so GTK stays happy even with WIN32-style paths
 *\param str string to fix the slashes
 *\return a string with slashes adapted to the needs of the OS
 */
std::string fixPathSlashes(const std::string &str);

/**
 * Trim off whitespace on the right side of the string
 *
 *\param str string to trim
 *\return trimmed string
 */
std::string trim_right(const std::string &str);

/**
 * Trim off the whitespace on the left side of the string
 *
 *\param str string to trim
 *\return trimmed string
 */
std::string trim_left(const std::string &str);

/**
 * Trim off the whitespace on both sides of the string
 *
 *\param str string to trim
 *\return trimmed string
 */
std::string trim_both(const std::string &str);

/**
 * Get the full path to the config file
 *
 * With some cross-os magic to support unix/win32/apple
 * borrowed from sear
 *
 *\return The complete path to the config file
 */
std::string getConfigFile();

#endif
