#include "glView.h"

glView::glView() : timer(), mNear(1.0f), mFar(1000.0f)
{
  Glib::RefPtr<Gdk::GL::Config> glconfig;

  bgColor[0] = bgColor[1] = bgColor[2] = 0.5;
	
  glconfig = Gdk::GL::Config::create(Gdk::GL::MODE_RGB | Gdk::GL::MODE_DEPTH  |
                                     Gdk::GL::MODE_DOUBLE);
  /**
   *\bug double buffer check removed
   */
#undef UNDEFINED
#ifdef UNDEFINED
  if (!glconfig.is_double_buffered())
    {
      std::cerr << "*** Cannot find the double-buffered visual.\n"
                << "*** Using single-buffered visual.\n";
    }
#endif
  set_gl_capability(glconfig);
}

glView::~glView()
{
}

void glView::calculate_adjustments() {
  adjW = 2.0f / get_width();
  adjH = 2.0f / get_height();
}

void glView::on_realize()
{
  DrawingArea::on_realize();

  Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();

  if (!glwindow->gl_begin(get_gl_context()))
    return;

  static GLfloat light0_diffuse[] = {1.0, 1.0, 1.0, 1.0};
  static GLfloat light0_specular[] = {1.0, 1.0, 1.0, 1.0};
  static GLfloat light0_position[] = {100.0, 100.0, 100.0, 0.0};
  static GLfloat light1_diffuse[] = {0.3, 0.3, 0.3, 1.0};
  static GLfloat light1_position[] = {-100.0, -100.0, 100.0, 0.0};
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
  glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
  glLightfv(GL_LIGHT1, GL_POSITION, light1_position);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_DEPTH_TEST);

  //	glClearColor(0.06, 0.23, 0.36, 1.0);
  glClearColor(bgColor[0], bgColor[1], bgColor[2], 1.0);
  glClearDepth(1.0);

  glViewport(0, 0, get_width(), get_height());

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, (GLfloat)get_width()/(GLfloat)get_height(), mNear, mFar);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glwindow->gl_end();

  calculate_adjustments();

  timer.start();
}

bool glView::on_configure_event(GdkEventConfigure* event)
{
  Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();

  if (!glwindow->gl_begin(get_gl_context()))
    return false;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, (GLfloat)get_width()/(GLfloat)get_height(), mNear, mFar);
  glViewport(0, 0, get_width(), get_height());
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
	
  glwindow->gl_end();

  calculate_adjustments();
  return true;
}

bool glView::on_expose_event(GdkEventExpose* event)
{
  Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();

  if (!glwindow->gl_begin(get_gl_context()))
    return false;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  double seconds = timer.elapsed();
	
  signal_render(seconds);
  if (glwindow->is_double_buffered())
    glwindow->swap_buffers();
  else
    glFlush();

  glwindow->gl_end();

  return true;
}

bool glView::on_map_event(GdkEventAny* event) {
  if(!connectionIdle.connected()) {
    connectionIdle = Glib::signal_idle().connect(
       sigc::mem_fun(*this, &glView::on_idle), GDK_PRIORITY_REDRAW);
  }
	
  return true;
}

bool glView::on_unmap_event(GdkEventAny* event) {
  if(connectionIdle.connected()) {
    connectionIdle.disconnect();
  }
  return true;
}

bool glView::on_visibility_notify_event(GdkEventVisibility* event) {
  if(event->state == GDK_VISIBILITY_FULLY_OBSCURED) {
    if(connectionIdle.connected()) {
      connectionIdle.disconnect();
    }
  } else {
    if(!connectionIdle.connected()) {
      connectionIdle = Glib::signal_idle().connect(
                                                   sigc::mem_fun(*this, &glView::on_idle), GDK_PRIORITY_REDRAW);
    }
  }
  return true;
}

bool glView::on_idle() {
  queue_draw();
  return true;
}

void glView::update() {
  on_idle();
}

void glView::setPlanes(const float nearplane, const float farplane) {
  mNear = nearplane;
  mFar = farplane;
  on_configure_event(NULL);
}

void glView::setBackgroundColor(double r, double g, double b) {

  bgColor[0] = r;
  bgColor[1] = g;
  bgColor[2] = b;
	
  Glib::RefPtr<Gdk::GL::Window> glwindow = get_gl_window();
	
  if (!glwindow->gl_begin(get_gl_context()))
    return;
  glClearColor(r, g, b, 1.0);
  glwindow->gl_end();
	
}

void glView::doPerspective() {
  gluPerspective(45.0, (GLfloat)get_width()/(GLfloat)get_height(), mNear, mFar);
}

void glView::getBackgroundColor(double *color) {
  color[0] = bgColor[0];
  color[1] = bgColor[1];
  color[2] = bgColor[2];
}
