#ifndef _EF_GLVIEW_H_
#define _EF_GLVIEW_H_

#include <glibmm/refptr.h>
#include <gtkmm/widget.h>
#include <gtkmm/drawingarea.h>

#include <gtkglmm.h>

#ifdef G_OS_WIN32
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#endif

#include <sigc++/object_slot.h>
#include <iostream>

#include <GL/gl.h>
#include <GL/glu.h>

#include <glibmm/signalproxy.h>
#include <glibmm/timer.h>

/**
 * The OpenGL widget that displays the model
 *
 * This is the OpenGL that will display the model. The class takes over
 * doing all the GL setup, etc.
 *
 *\author Henti Kuuste
 */

class glView : public Gtk::GL::DrawingArea
{
 public:
  /**
   * Default constructor
   */
  glView();
	
  /**
   * Default destructor
   */
  virtual ~glView();
  /**
   * Set the GL near and far planes
   *
   * Make sure differently scaled objects aren't cut off. This will be
   * checked every time the model is loaded or resized
   *\param nearplane 
   *\param farplane
   */
  void setPlanes(const float nearplane, const float farplane);
  /**
   * Set the background color
   *
   * Sets the OpenGL widget's backgroun color
   *\param r red
   *\param g green
   *\param b blue
   *\todo comment the real function
   */
  void setBackgroundColor(double r, double g, double b);
  /**
   * Get the background color
   *
   *\param color a pointer to a double[3] array that takes the rgb values
   */
  void getBackgroundColor(double *color);
	
  /**
   * ???
   */
  sigc::signal1<void, double> signal_render;
  Glib::Timer timer;

  /**
   * What to do when update() is called on this widget
   *
   * Actually all that's done is calling on_idle()
   *\see #on_idle()
   */
  void update();

  /**
   * Sets the perspective
   */
  void doPerspective();
  
  /**
   * Calculate the adjusted width and height.
   */
  void calculate_adjustments();

  GLfloat adjW;
  GLfloat adjH;
	
 protected:
  virtual void on_realize();
  virtual bool on_configure_event(GdkEventConfigure* event);
  virtual bool on_expose_event(GdkEventExpose* event);
  virtual bool on_map_event(GdkEventAny* event);
  virtual bool on_unmap_event(GdkEventAny* event);
  virtual bool on_visibility_notify_event(GdkEventVisibility* event);
  virtual bool on_idle();
	
  sigc::connection connectionIdle;
  float mNear, mFar;
  double bgColor[3];

  //	int frames;
};

#endif // _EF_GLVIEW_H_
