// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2004 Kai Blin

#include "mainWindow.h"
#include "skull_n_crossbones.xpm"
#include "wireframe.xpm"

void mainWindow::createGUI() {
  //I get two valgrind errors from this.  Why?
  mainWin = new Gtk::Window();
  mainWin->set_title("EntityForge - The WorldForge Entity Composer");
  mainWin->set_default_size(800, 600);
  mainWin->set_resizable();
  mainWin->signal_motion_notify_event().connect(sigc::mem_fun(*this,&mainWindow::on_main_mouse_move));
  mainWin->signal_key_press_event().connect_notify(sigc::mem_fun(*this, 
                                                                 &mainWindow::on_main_key_press));
  mainWin->signal_key_release_event().connect_notify(sigc::mem_fun(*this, 
                                                                   &mainWindow::on_main_key_release));
  createAboutDialog();
  createControlsHelpDialog();

  Gtk::VBox *mainBox = new Gtk::VBox();
  mainWin->add(*Gtk::manage(mainBox));

  mainBox->pack_start(*Gtk::manage(createMenu()), Gtk::PACK_SHRINK);
  Gtk::HBox *toolbarBox = new Gtk::HBox();
  mainBox->pack_start(*Gtk::manage(toolbarBox), Gtk::PACK_SHRINK);
  toolbarBox->pack_start(*Gtk::manage(createTopToolbar()), Gtk::PACK_EXPAND_WIDGET);
  toolbarBox->pack_start(*Gtk::manage(createToolbarTable()), Gtk::PACK_SHRINK);

  //And on the left side, we want a box, with the models at the top, and the tree at the bottom

  treePane = new Gtk::HPaned();
  mainBox->pack_start(*Gtk::manage(treePane), Gtk::PACK_EXPAND_WIDGET);
  treePane->set_position(50);

  Gtk::VPaned *leftBox = new Gtk::VPaned();
  treePane->pack1(*Gtk::manage(leftBox), Gtk::SHRINK);
	
  Gtk::ScrolledWindow *modelsScroll = new Gtk::ScrolledWindow();
  leftBox->pack1(*Gtk::manage(modelsScroll), Gtk::SHRINK);
  modelsScroll->add(*Gtk::manage(createModelsView()));
  modelsScroll->set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);
	
  Gtk::ScrolledWindow *treeScroll = new Gtk::ScrolledWindow();
  leftBox->pack2(*Gtk::manage(treeScroll), Gtk::EXPAND);
  //treePane->pack1(*Gtk::manage(treeScroll), Gtk::SHRINK);
  treeScroll->add(*Gtk::manage(createTreeView()));
  treeScroll->set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);

  infoPane = new Gtk::VPaned();
  treePane->pack2(*Gtk::manage(infoPane), Gtk::EXPAND);
  infoPane->set_position(400);
  Gtk::HBox *viewportBox = new Gtk::HBox();
  infoPane->pack1(*Gtk::manage(viewportBox), Gtk::EXPAND);
  viewportBox->pack_start(*Gtk::manage(createViewport()), Gtk::PACK_EXPAND_WIDGET);
  viewportBox->pack_start(*Gtk::manage(createRightToolbar()), Gtk::PACK_SHRINK);
	
  Gtk::ScrolledWindow *infoScroll = new Gtk::ScrolledWindow();
  infoScroll->set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC);
  infoPane->pack2(*Gtk::manage(infoScroll), Gtk::EXPAND);
  infoBox = new Gtk::VBox();
  infoScroll->add(*Gtk::manage(infoBox));

  mainBox->pack_start(*Gtk::manage(createBottomInfo()), Gtk::PACK_SHRINK);
	
  helpBar = new Gtk::Statusbar();
  mainBox->pack_start(*Gtk::manage(helpBar), Gtk::PACK_SHRINK);
  createExecuteActionDialog();
  fprintf(stderr, "Finished making GUI! Showing..\n");
  mainWin->show_all();
  fprintf(stderr, "\n\n\n*****done*****\n");
}

Gtk::MenuBar *mainWindow::createMenu() {
  Gtk::MenuBar *theMenu = new Gtk::MenuBar();
  theMenu->accelerate(*mainWin);
	
  Gtk::MenuItem *pMenuItem = new Gtk::MenuItem("_File", true);
  pMenuItem->set_submenu(*Gtk::manage(createFileMenu()));
  theMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("_Edit", true);
  pMenuItem->set_submenu(*Gtk::manage(createEditMenu()));
  theMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("_View", true);
  pMenuItem->set_submenu(*Gtk::manage(createViewMenu()));
  theMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("_Options", true);
  pMenuItem->set_submenu(*Gtk::manage(createOptionsMenu()));
  theMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("_Help", true);
  pMenuItem->set_submenu(*Gtk::manage(createHelpMenu()));
  theMenu->append(*Gtk::manage(pMenuItem));

  return theMenu;
}

Gtk::Menu *mainWindow::createFileMenu() {
  Gtk::Menu *m = new Gtk::Menu();
  Gtk::ImageMenuItem *pIMI;
  Gtk::MenuItem *pMI;
  Gtk::Image *pI;
  Gtk::SeparatorMenuItem *pSMI;
	
  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::NEW);
  pIMI->signal_activate().connect(
                                  sigc::mem_fun(*this, &mainWindow::on_new));
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));
	
  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::OPEN);
  pIMI->signal_activate().connect(
                                  sigc::mem_fun(*this, &mainWindow::on_open));	
  m->append(*Gtk::manage(pIMI));
	
  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::SAVE);
  pIMI->signal_activate().connect(
                                  sigc::mem_fun(*this, &mainWindow::on_save));	
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));

  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::SAVE_AS);
  pIMI->signal_activate().connect(
                                  sigc::mem_fun(*this, &mainWindow::on_save_as));	
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));
	
  pSMI = new Gtk::SeparatorMenuItem();
  m->append(*Gtk::manage(pSMI));

  pI = new Gtk::Image(Gtk::Stock::REDO, Gtk::IconSize(1));
  pIMI = new Gtk::ImageMenuItem(*Gtk::manage(pI), "Import");
  pIMI->signal_activate().connect(sigc::mem_fun(*this, &mainWindow::on_import));
  m->append(*Gtk::manage(pIMI));

  pI = new Gtk::Image(Gtk::Stock::UNDO, Gtk::IconSize(1));
  pIMI = new Gtk::ImageMenuItem(*Gtk::manage(pI), "Export");
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));

  Gtk::Menu *subMenu = new Gtk::Menu();
  pIMI->set_submenu(*Gtk::manage(subMenu));
	
  pMI = new Gtk::MenuItem("Cal3D cfg");
  pMI->signal_activate().connect(
                                 sigc::mem_fun(*this, &mainWindow::on_cal3d_cfg_export));
  pIMI->set_sensitive(false);
  subMenu->append(*Gtk::manage(pMI));
	
  pSMI = new Gtk::SeparatorMenuItem();
  m->append(*Gtk::manage(pSMI));

  pMI = new Gtk::MenuItem("Recent Files");
  m->append(*Gtk::manage(pMI));
  recentFilesMenu = new Gtk::Menu();
  pMI->set_submenu(*Gtk::manage(recentFilesMenu));

  pSMI = new Gtk::SeparatorMenuItem();
  m->append(*Gtk::manage(pSMI));

  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::QUIT);
  pIMI->signal_activate().connect(
                                  sigc::mem_fun(*mainWin, &Gtk::Window::hide));
  m->append(*Gtk::manage(pIMI));
	
  return m;
}

Gtk::Menu *mainWindow::createEditMenu() {
  Gtk::Menu *m = new Gtk::Menu();
  Gtk::ImageMenuItem *pIMI;
	
  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::CUT);
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));
	
  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::COPY);
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));
	
  pIMI = new Gtk::ImageMenuItem(Gtk::Stock::PASTE);
  pIMI->set_sensitive(false);
  m->append(*Gtk::manage(pIMI));
	
  Gtk::MenuItem *ins = new Gtk::MenuItem("Insert");
  m->append(*Gtk::manage(ins));
  Gtk::Menu *subMenu = new Gtk::Menu();
  ins->set_submenu(*Gtk::manage(subMenu));
	
  Gtk::MenuItem *pMenuItem = new Gtk::MenuItem("Standard static object");
  pMenuItem->signal_activate().connect(sigc::mem_fun(*this, &mainWindow::on_static_object));
  subMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem(".obj");
  pMenuItem->signal_activate().connect(sigc::mem_fun(*this, &mainWindow::on_obj));
  subMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("MD3");
  pMenuItem->signal_activate().connect(sigc::mem_fun(*this, &mainWindow::on_md3));
  subMenu->append(*Gtk::manage(pMenuItem));
	
  return m;
}

Gtk::Menu *mainWindow::createViewMenu() {
  Gtk::Menu *m = new Gtk::Menu();

  Gtk::MenuItem *pMI = new Gtk::MenuItem("Reset Transformations");
  pMI->signal_activate().connect(
                                 sigc::mem_fun(*this, &mainWindow::reset_transforms));
  m->append(*Gtk::manage(pMI));
	
  return m;
}

Gtk::Menu *mainWindow::createOptionsMenu() {
  Gtk::Menu *m = new Gtk::Menu();

  wireOnSurface = new Gtk::CheckMenuItem("Wireframe on surface");
  wireOnSurface->signal_toggled().connect(
                                          sigc::mem_fun(*this, &mainWindow::on_wire_on_surface));
  m->append(*Gtk::manage(wireOnSurface));
	
  backfaceCulling = new Gtk::CheckMenuItem("Backface culling");
  backfaceCulling->signal_toggled().connect(
                                            sigc::mem_fun(*this, &mainWindow::on_backface_culling));
  m->append(*Gtk::manage(backfaceCulling));

  Gtk::MenuItem *frontFace = new Gtk::MenuItem("Front face");
  m->append(*Gtk::manage(frontFace));
  Gtk::Menu *subMenu = new Gtk::Menu();
  frontFace->set_submenu(*Gtk::manage(subMenu));
	
  Gtk::RadioMenuItem::Group g;

  cw = new Gtk::RadioMenuItem(g, "clockwise");
  cw->signal_toggled().connect(
                               sigc::mem_fun(*this, &mainWindow::on_menu_cw));
  subMenu->append(*Gtk::manage(cw));
  cw->set_active(false);
	
  ccw = new Gtk::RadioMenuItem(g, "counter clockwise");
  cw->signal_toggled().connect(
                               sigc::mem_fun(*this, &mainWindow::on_menu_ccw));
  subMenu->append(*Gtk::manage(ccw));
  ccw->set_active(true);
	
  Gtk::MenuItem *prefs = new Gtk::MenuItem("Preferences");
  m->append(*Gtk::manage(prefs));
  subMenu = new Gtk::Menu();
  prefs->set_submenu(*Gtk::manage(subMenu));
	
  Gtk::MenuItem *pMenuItem = new Gtk::MenuItem("Background color");
  pMenuItem->signal_activate().connect(
                                       sigc::mem_fun(*this, &mainWindow::on_background_color));
  subMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("Selection color");
  pMenuItem->signal_activate().connect(
                                       sigc::mem_fun(*this, &mainWindow::on_selection_color));
  subMenu->append(*Gtk::manage(pMenuItem));

  pMenuItem = new Gtk::MenuItem("Camera Rotation Mode");
  subMenu->append(*Gtk::manage(pMenuItem));
  Gtk::Menu *subMenu2 = new Gtk::Menu();
  pMenuItem->set_submenu(*Gtk::manage(subMenu2));
	
  Gtk::RadioMenuItem::Group rm;

  mode_arcball = new Gtk::RadioMenuItem(rm, "ArcBall");
  mode_arcball->signal_toggled().connect(sigc::bind(
                                                    sigc::mem_fun(*this, &mainWindow::set_rotMode),
                                                    ARCBALL, false));
  subMenu2->append(*Gtk::manage(mode_arcball));
  mode_arcball->set_active(true);
	
  mode_euler = new Gtk::RadioMenuItem(rm, "Euler Angles");
  mode_euler->signal_toggled().connect(sigc::bind(
                                                  sigc::mem_fun(*this, &mainWindow::set_rotMode),
                                                  EULER, false));
  subMenu2->append(*Gtk::manage(mode_euler));
  mode_euler->set_active(true);
  return m;
}

Gtk::Menu *mainWindow::createHelpMenu() {
  Gtk::Menu *m = new Gtk::Menu();
  Gtk::MenuItem *pMI;

  pMI = new Gtk::MenuItem("Controls");
  pMI->signal_activate().connect(
                                 sigc::mem_fun(*controlsHelpDialog, &Gtk::Dialog::show));
  m->append(*Gtk::manage(pMI));
	
  pMI = new Gtk::MenuItem("About");
  pMI->signal_activate().connect(
                                 sigc::mem_fun(*aboutDialog, &Gtk::Dialog::show));
  m->append(*Gtk::manage(pMI));
	
  return m;
}

Gtk::Toolbar *mainWindow::createTopToolbar() {
  Gtk::Toolbar *t = new Gtk::Toolbar();
  t->set_toolbar_style(Gtk::TOOLBAR_ICONS);
  t->set_tooltips();
  t->set_orientation(Gtk::ORIENTATION_HORIZONTAL);

  Gtk::ToolButton* new_button = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::NEW));
  new_button->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_new));
  new_button->set_sensitive(false);
  t->append(*new_button);

  Gtk::ToolButton* open_button = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::OPEN));
  open_button->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_open));
  open_button->set_sensitive(false);
  t->append(*open_button);

  Gtk::ToolButton* save_button = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::SAVE));
  save_button->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_save));
  save_button->set_sensitive(false);
  t->append(*save_button);

  Gtk::ToolButton* save_as_button = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::SAVE_AS));
  save_as_button->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_save_as));
  save_as_button->set_sensitive(false);
  t->append(*save_as_button);

  return t;
}

Gtk::Table *mainWindow::createToolbarTable() {
  Gtk::Table *pTab = new Gtk::Table(3, 3);
  Gtk::Label *pLabel = new Gtk::Label("LOD: ");
  assert(pTab);
  assert(pLabel);
  pLabel->property_xalign().set_value(0.0);
  pTab->attach(*Gtk::manage(pLabel),
               0, 1, 0, 1, Gtk::FILL, Gtk::FILL, 4);

  Gtk::Adjustment *pAd = new Gtk::Adjustment(100, 0, 100, 1, 10, 0.0);
  lodScale = new Gtk::HScale(*Gtk::manage(pAd));
  lodScale->set_size_request(275,-1);
  lodScale->set_update_policy(Gtk::UPDATE_CONTINUOUS);
  lodScale->set_digits(0);
  lodScale->set_draw_value(false);
  lodScale->signal_value_changed().connect(sigc::mem_fun(*this, &mainWindow::on_lod));

  //Crashes when we add a spin box - don't know why.
  //Both here and below
  Gtk::SpinButton *pSpin = new Gtk::SpinButton(*Gtk::manage(pAd), 1.0, 0);
	
  pTab->attach(*Gtk::manage(pSpin),
               1, 2, 0, 1, Gtk::SHRINK, Gtk::FILL);
  pTab->attach(*Gtk::manage(lodScale),
               2, 3, 0, 1, Gtk::EXPAND | Gtk::FILL, Gtk::FILL);

  //error before here	
  pLabel = new Gtk::Label("Time scale: ");
  pLabel->property_xalign().set_value(0.0);
  pTab->attach(*Gtk::manage(pLabel),
               0, 1, 1, 2, Gtk::FILL, Gtk::FILL, 4);
	
  pAd = new Gtk::Adjustment(1, 0, 5, 0.1, 0.5, 0.0);
  timeScale = new Gtk::HScale(*Gtk::manage(pAd));
  timeScale->set_size_request(275,-1);
  timeScale->set_update_policy(Gtk::UPDATE_CONTINUOUS);
  timeScale->set_digits(0);
  timeScale->set_draw_value(false);
  Gtk::SpinButton *pSpin2 = new Gtk::SpinButton(*Gtk::manage(pAd), 1.0, 1);
  pTab->attach(*Gtk::manage(pSpin2),
               1, 2, 1, 2, Gtk::SHRINK, Gtk::FILL);
  pTab->attach(*Gtk::manage(timeScale),
               2, 3, 1, 2, Gtk::EXPAND | Gtk::FILL, Gtk::FILL);


  // add another label spinbutton hscale, this time for the model's scale
  Gtk::Label *scale_label = new Gtk::Label("Model scale: ");
  Gtk::Adjustment *scale_adjustment = new Gtk::Adjustment(0.0, -4.0, 1.0, 0.1, 0.5, 0.0);
  Gtk::SpinButton *scale_spinbutton = new Gtk::SpinButton(*Gtk::manage(scale_adjustment), 1.0, 1);

  size_scale = new Gtk::HScale(*Gtk::manage(scale_adjustment));
  size_scale->set_size_request(275, -1);
  size_scale->set_update_policy(Gtk::UPDATE_CONTINUOUS);
  size_scale->set_digits(0);
  size_scale->set_draw_value(false);
  size_scale->signal_value_changed().connect(sigc::mem_fun(*this, &mainWindow::on_sizeScale));

  pTab->attach(*Gtk::manage(scale_label),
               0, 1, 2, 3, Gtk::FILL, Gtk::FILL, 4);
  pTab->attach(*Gtk::manage(scale_spinbutton),
               1, 2, 2, 3, Gtk::SHRINK, Gtk::SHRINK);
  pTab->attach(*Gtk::manage(size_scale),
               2, 3, 2, 3, Gtk::EXPAND | Gtk::FILL, Gtk::FILL);
  
  return pTab;
}

Gtk::Toolbar *mainWindow::createRightToolbar() {
  Gtk::Toolbar *t = new Gtk::Toolbar();
  t->set_orientation(Gtk::ORIENTATION_VERTICAL);
  t->set_tooltips();
  t->set_toolbar_style(Gtk::TOOLBAR_ICONS);

  Gtk::ToolButton* refresh = Gtk::manage(new Gtk::ToolButton(Gtk::Stock::REFRESH));
  refresh->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::refresh));
  t->append(*refresh);

  Gtk::ToggleToolButton* lights = Gtk::manage(new Gtk::ToggleToolButton(Gtk::Stock::DIALOG_INFO));
  lights->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_lights));
  lights->set_active(true);
  t->append(*lights);

  //  t->append(Gtk::Toolbar_Helpers::ToggleElem("lights",
  //                                           *Gtk::manage(new Gtk::Image(Gtk::Stock::DIALOG_INFO, Gtk::ICON_SIZE_LARGE_TOOLBAR)),
  //                                           sigc::mem_fun(*this, &mainWindow::on_lights),
  //                                             "Turn lights on/off"));

  
  select_button = Gtk::manage(new Gtk::ToggleToolButton(Gtk::Stock::INDEX));
  select_button->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_select));
  t->append(*select_button);

  try {
    Glib::RefPtr<Gdk::Pixbuf> pb = Gdk::Pixbuf::create_from_xpm_data(skull_n_crossbones_xpm);
    Gtk::Image *pI = new Gtk::Image(pb);
    
    Gtk::ToggleToolButton* skeleton = Gtk::manage(new Gtk::ToggleToolButton(*Gtk::manage(pI)));
    skeleton->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_skeleton));
    t->append(*skeleton);

    //    t->append(Gtk::Toolbar_Helpers::ToggleElem("skeleton",
    //                                           *Gtk::manage(pI),
    //                                           sigc::mem_fun(*this, &mainWindow::on_skeleton),
    //                                           "Turn skeleton rendering on/off"));
    
    pb = Gdk::Pixbuf::create_from_xpm_data(wireframe_xpm);
    pI = new Gtk::Image(pb);

    Gtk::ToggleToolButton* wireframe = Gtk::manage(new Gtk::ToggleToolButton(*Gtk::manage(pI)));
    wireframe->signal_clicked().connect(sigc::mem_fun(*this, &mainWindow::on_wireframe));
    t->append(*wireframe);

    //    t->append(Gtk::Toolbar_Helpers::ToggleElem("wireframe",
    //                                           *Gtk::manage(pI),
    //                                           sigc::mem_fun(*this, &mainWindow::on_wireframe),
    //                                           "Turn wireframe rendering on/off"));
	
  } catch (Gdk::PixbufError e) {
    fprintf(stderr, "Could not load images for toolbar.  Aborting sorry.  Files should be in INSTALLDIR/share/\n");
    cout << e.code() << endl;
    exit(EXIT_FAILURE);
  } catch (Glib::FileError fe) {
    fprintf(stderr, "Could not load images for toolbar.  Aborting sorry.  Try running this in the directory that has the Data directory.\n");
    cout << fe.code() << endl;
    exit(EXIT_FAILURE);
  }

  return t;
}

void mainWindow::createAboutDialog() {
  aboutDialog = new Gtk::Dialog("About EntityForge", *mainWin, true);
  Gtk::Button *pButton = aboutDialog->add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  aboutDialog->get_action_area()->set_layout(Gtk::BUTTONBOX_SPREAD);
  aboutDialog->property_window_position().set_value(Gtk::WIN_POS_CENTER_ON_PARENT);
  aboutDialog->set_transient_for(*mainWin);
  Gtk::Label *pLabel = new Gtk::Label(
                                      "EntityForge Prerelease 1.1\n"
                                      "This is EntityForge, the entity editor/viewer for Worldforge.\n"
                                      "Author: Henri Kuuste aka RTSan\n"
                                      "<rtsan@hot.ee>\n"
                                      "Maintainer: Kai Blin aka nowhere\n"
                                      "<blin@gmx.net>\n"
                                      "\n"
                                      "Windows Port: Jonathan Giszczak aka DragonM\n"
                                      "<dragonm@hypercubepc.com>\n"
                                      "Licence: GPL (see COPYING for details)\n"
                                      "\n"
                                      "Worldforge\n"
                                      "www.worldforge.org\n"
                                      "irc.worldforge.org");
  pLabel->set_alignment(0.5,0.5);
  pLabel->set_padding(0,0);
  pLabel->set_justify(Gtk::JUSTIFY_CENTER);
  aboutDialog->get_vbox()->pack_start(*Gtk::manage(pLabel));
  pButton->signal_clicked().connect(
                                    sigc::mem_fun(*aboutDialog, &Gtk::Dialog::hide));
  aboutDialog->get_vbox()->show_all();
}

void mainWindow::createControlsHelpDialog() {
  controlsHelpDialog = new Gtk::Dialog("Controls", *mainWin, true);
  Gtk::Button *pButton = controlsHelpDialog->add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  controlsHelpDialog->get_action_area()->set_layout(Gtk::BUTTONBOX_SPREAD);
  controlsHelpDialog->property_window_position().set_value(Gtk::WIN_POS_CENTER_ON_PARENT);
  controlsHelpDialog->set_transient_for(*mainWin);
  Gtk::Label *pLabel = new Gtk::Label("The controls for the viewport are as follows:\n"
                                      "Rotate\t -\t MMB Drag\n"
                                      "Zoom\t -\t CTRL+MMB Drag\n"
                                      "Translate\t -\t SHIFT+MMB Drag\n"
                                      "Select\t -\t SHIFT+LMB\n"
                                      "\n"
                                      "There is also a context sensitive menu on RMB.\n"
                                      "Currently this menu is pretty poor\n"
                                      "In the viewport you get a general menu and it should change based on the selection\n"
                                      "However, only the submesh selection has something in there\n"
                                      "Options are also available on submeshes and actions in the tree\n"
                                      "Double clicking on an action executes it as 'Fast Execute' from the menu would\n"
                                      "\n"
                                      "Animation blending can be controlled by selecting Cycles from the tree\n"
                                      "Also by selecting individual cycles\n"
                                      "Double clicking on a cycle causes that cycle to be activated with a blend of 1\n");
  pLabel->set_alignment(0.0,0.5);
  pLabel->set_padding(0,0);
  pLabel->set_justify(Gtk::JUSTIFY_LEFT);
  controlsHelpDialog->get_vbox()->pack_start(*Gtk::manage(pLabel));
  pButton->signal_clicked().connect(
                                    sigc::mem_fun(*controlsHelpDialog, &Gtk::Dialog::hide));
  controlsHelpDialog->get_vbox()->show_all();
}

void mainWindow::createExecuteActionDialog() {
  executeActionDialog = new Gtk::Dialog("Execute Action", *mainWin, true);
  executeActionDialog->get_action_area()->set_layout(Gtk::BUTTONBOX_SPREAD);
  executeActionDialog->property_window_position().set_value(Gtk::WIN_POS_CENTER_ON_PARENT);
  executeActionDialog->set_transient_for(*mainWin);
  Gtk::Button *pButton = executeActionDialog->add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  pButton->signal_clicked().connect(
                                    sigc::mem_fun(*executeActionDialog, &Gtk::Dialog::hide));
  pButton = executeActionDialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  pButton->signal_clicked().connect(
                                    sigc::mem_fun(*executeActionDialog, &Gtk::Dialog::hide));

  Gtk::Table *pTab = new Gtk::Table(2, 2);
  executeActionDialog->get_vbox()->pack_start(*Gtk::manage(pTab));
  pTab->attach(*Gtk::manage(new Gtk::Label("Fade in delay: ")),
               0, 1, 0, 1, Gtk::FILL, Gtk::FILL, 4, 4);
  Gtk::Adjustment *pAd = new Gtk::Adjustment(0.5, 0, 10, 0.1, 0.5, 1);
  fadeInDelaySpin = new Gtk::SpinButton(*Gtk::manage(pAd), 1, 1);
  pTab->attach(*Gtk::manage(fadeInDelaySpin),
               1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);

  pTab->attach(*Gtk::manage(new Gtk::Label("Fade out delay: ")),
               0, 1, 1, 2, Gtk::FILL, Gtk::FILL, 4, 4);
  pAd = new Gtk::Adjustment(0.5, 0, 10, 0.1, 0.5, 1);
  fadeOutDelaySpin = new Gtk::SpinButton(*Gtk::manage(pAd), 1, 1);
  pTab->attach(*Gtk::manage(fadeOutDelaySpin),
               1, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
  executeActionDialog->get_vbox()->show_all();
}

Gtk::EventBox *mainWindow::createViewport() {

  viewportFrame = new Gtk::EventBox();
  viewport = Gtk::manage(new glView);
  viewportFrame->add(*viewport);
  viewport->signal_render.connect(
                                  sigc::mem_fun(*this, &mainWindow::renderGL));
  viewportFrame->show_all();
  viewportFrame->signal_button_press_event().connect(
                                                     sigc::mem_fun(*this, &mainWindow::on_viewport_button_press));
  viewportFrame->signal_button_release_event().connect(
                                                       sigc::mem_fun(*this, &mainWindow::on_viewport_button_release));
  viewportFrame->signal_motion_notify_event().connect(
                                                      sigc::mem_fun(*this, &mainWindow::on_viewport_mouse_move));
	
  return viewportFrame;
}

Gtk::TreeView *mainWindow::createTreeView() {
  entityTree = new Gtk::TreeView();
  entityTree->set_headers_visible(false);
  entityTree->set_rules_hint(true);
  entityTree->signal_event().connect(sigc::mem_fun(*this, &mainWindow::on_tree_event), true);
  refTreeSelection = entityTree->get_selection();
  refTreeSelection->signal_changed().connect(
                                             sigc::mem_fun(*this, &mainWindow::on_tree_selection_changed));	
  return entityTree;
}

Gtk::HBox *mainWindow::createBottomInfo() {
  Gtk::HBox *b = new Gtk::HBox();
  b->pack_start(*Gtk::manage(new Gtk::Label("Polycount: ")), Gtk::PACK_SHRINK);
  std::ostringstream str;
  str << getPolyCount();
  polyCount = new Gtk::Label(str.str());
  polyCount->property_xalign().set_value(0.0);
  polyCount->set_justify(Gtk::JUSTIFY_LEFT);
  polyCount->set_size_request(40,-1);
  b->pack_start(*Gtk::manage(polyCount), Gtk::PACK_SHRINK);
  b->pack_start(*Gtk::manage(new Gtk::VSeparator()), Gtk::PACK_SHRINK, 5);
  b->pack_start(*Gtk::manage(new Gtk::Label("FPS: ")), Gtk::PACK_SHRINK);
  fps = new Gtk::Label("0");
  fps->property_xalign().set_value(0.0);
  fps->set_justify(Gtk::JUSTIFY_LEFT);
  b->pack_start(*Gtk::manage(fps), Gtk::PACK_EXPAND_WIDGET);	
  return b;
}


Gtk::Table *mainWindow::createModelsView() {
  Gtk::Table *tTable = new Gtk::Table(2,5);

  Gtk::Label *tLabel = new Gtk::Label("Model: ");
  //	tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 0, 1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  Gtk::Combo *tCombo = new Gtk::Combo();
	
  tCombo->set_value_in_list(false,false);
  std::vector<Glib::ustring> items;
  items.push_back("Skeleton");
  items.push_back("Paladin");
  items.push_back("Human Male");
  tCombo->set_popdown_strings(items);
  tTable->attach(*Gtk::manage(tCombo),
                 1,2,0,1,Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK);
  return tTable;
}


Gtk::Table *mainWindow::createCyclesInfoTable(Gtk::TreeModel::iterator iter) {
  Gtk::TreeModel::Row row = *iter;
  Gtk::TreeModel::Children cycles = row.children();
  Gtk::Table *tTable = new Gtk::Table(cycles.size()+1, 4);
  tTable->attach(*Gtk::manage(new Gtk::Label("Cycle Blending")),
                 0, 4, 0, 1, Gtk::SHRINK, Gtk::SHRINK);
  int j = 1;
  for(Gtk::TreeModel::Children::iterator iter = cycles.begin(); iter != cycles.end(); ++iter)
    {
      Gtk::TreeModel::Row cycle = *iter;
      int cycleId = cycle[cols.id];
      Gtk::CheckButton *tCheck = new Gtk::CheckButton();
      tCheck->set_active(cycle[cols.active]);
      tCheck->signal_toggled().connect(
                                       sigc::bind(
                                                  sigc::mem_fun(*this, &mainWindow::on_cycle_toggle),
                                                  iter));
      tTable->attach(*Gtk::manage(tCheck),
                     0, 1, j, j+1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 2);
      Gtk::Label *tLabel = new Gtk::Label(cycle[cols.text]);
      tLabel->property_xalign().set_value(0.0);
      tTable->attach(*Gtk::manage(tLabel),
                     1, 2, j, j+1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
      tTable->attach(*Gtk::manage(calm->animCycles[cycleId]->createSpinButton()),
                     2, 3, j, j+1, Gtk::SHRINK, Gtk::SHRINK);
      tTable->attach(*Gtk::manage(calm->animCycles[cycleId]->createScaleWidget()),
                     3, 4, j, j+1);
      j++;
    }
  return tTable;
}

Gtk::Table *mainWindow::createModelInfoTable(Gtk::TreeModel::iterator iter) {
  Gtk::Table *tTable = new Gtk::Table(3, 3);
  Gtk::Label *tLabel = new Gtk::Label("Path: ");
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 0, 1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  tTable->attach(*calm->dirEntry,
                 1, 2, 0, 1);
  tLabel = new Gtk::Label("Scale: ");
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 1, 2, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  tTable->attach(*Gtk::manage(calm->createScaleSpin()),
                 1, 2, 1, 2);
  tLabel = new Gtk::Label("Polycount: ");
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 2, 3, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  std::ostringstream pc;
  pc << calm->getPolyCount();
  pcLabel = new Gtk::Label(pc.str());
  pcLabel->property_xalign().set_value(0.0);
  pcLabel->set_justify(Gtk::JUSTIFY_LEFT);
  tTable->attach(*Gtk::manage(pcLabel),
                 1, 2, 2, 3);
  return tTable;
}

Gtk::Table *mainWindow::createMeshInfoTable(Gtk::TreeModel::iterator iter) {
  Gtk::Table *tTable = new Gtk::Table(1, 1);
  Gtk::Label *tLabel = new Gtk::Label("Polycount: ");
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 0, 1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  std::ostringstream pc;
  pc << calm->getMeshPolyCount((*iter)[cols.id]);
  pcLabel = new Gtk::Label(pc.str());
  pcLabel->property_xalign().set_value(0.0);
  pcLabel->set_justify(Gtk::JUSTIFY_LEFT);
  tTable->attach(*Gtk::manage(pcLabel),
                 1, 2, 0, 1);
  return tTable;
}

Gtk::Table *mainWindow::createSubmeshInfoTable(Gtk::TreeModel::iterator iter) {
  Gtk::TreeModel::Row row = *iter;
  Gtk::Table *tTable = new Gtk::Table(1, 1);
  Gtk::Label *tLabel = new Gtk::Label("Polycount: ");
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 0, 1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  std::ostringstream pc;
  pc << calm->getSubmeshPolyCount((*row.parent())[cols.id], row[cols.id]);
  pcLabel = new Gtk::Label(pc.str());
  pcLabel->property_xalign().set_value(0.0);
  pcLabel->set_justify(Gtk::JUSTIFY_LEFT);
  tTable->attach(*Gtk::manage(pcLabel),
                 1, 2, 0, 1);
  return tTable;
}

Gtk::Table *mainWindow::createMaterialInfoTable(Gtk::TreeModel::iterator iter) {
  Gtk::TreeModel::Row row = *iter;
  Gtk::Table *tTable = new Gtk::Table(1, 2);
  Gtk::Label *tLabel = new Gtk::Label("Path: ");
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 0, 1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  Gtk::Entry *tEntry = new Gtk::Entry();
  tEntry->set_text(calm->textureDirs[row[cols.id]]);
  tTable->attach(*Gtk::manage(tEntry),
                 1, 2, 0, 1);
  return tTable;
}

Gtk::Table *mainWindow::createCycleInfoTable(Gtk::TreeModel::iterator iter) {
  Gtk::Table *tTable = new Gtk::Table(1, 3);
  Gtk::Label *tLabel = new Gtk::Label("Blending");
  int id = (*iter)[cols.id];
  tLabel->property_xalign().set_value(0.0);
  tTable->attach(*Gtk::manage(tLabel),
                 0, 1, 0, 1, Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK, 4);
  tTable->attach(*Gtk::manage(calm->animCycles[id]->createScaleWidget()),
                 2, 3, 0, 1);
  tTable->attach(*Gtk::manage(calm->animCycles[id]->createSpinButton()),
                 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK);
  return tTable;
}

