// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2004 Kai Blin

#include "md3_model.h"

#ifdef HAVE_LIBMD3

Md3Model::Md3Model(): m_model_file(NULL){
}

Md3Model::Md3Model(CalBone *bone): StaticObject(bone) {
  Md3Model();
}

Md3Model::~Md3Model(){
  delete m_model_file;
}

bool Md3Model::readFile(std::string file){
  m_model_file = libmd3_file_load(file.c_str());
  if(m_model_file == NULL) {
    return false;
  }

  for(int i = 0; i < m_model_file->header->mesh_count; ++i){
    libmd3_unpack_normals(&m_model_file->meshes[i]);
  }

  return true;
}

void Md3Model::render() {
  std::cout << "Md3Model::render() called" << std::endl;
  glPushMatrix();
  applyTransformation();

  libmd3_mesh *meshptr;

  meshptr = m_model_file->meshes;
  for(int i = 0; i < m_model_file->header->mesh_count; ++i, ++meshptr){
    drawOneMesh(meshptr);
  }
  glPopMatrix();
}

void Md3Model::drawOneMesh(libmd3_mesh *mesh){
  if(mesh->mesh_header->skin_count != 0){
    if(mesh->user.u == 0) {
      char* skin_name = (char*)mesh->skins[0].name;
      std::string skin_name_str(skin_name);
      mesh->user.u = loadGTK(fixPathSlashes(skin_name_str));
    }
  }

  if (mesh->user.u != 0){
    glEnable(GL_TEXTURE_2D);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glBindTexture(GL_TEXTURE_2D, mesh->user.u);
  }

  glVertexPointer(3, GL_SHORT, 0, mesh->vertices);
  glTexCoordPointer(2, GL_FLOAT, 0, mesh->texcoords);
  glDrawElements(GL_TRIANGLES, mesh->mesh_header->triangle_count * 3,
                 GL_UNSIGNED_INT, mesh->triangles);

  if (mesh->user.u != 0) {
    glDisable(GL_TEXTURE_2D);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  }
}

#endif
