// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2003 Henri Kuuste

#include "static_object.h"

StaticObject::StaticObject() : ball(CalVector(0, 0, 0), 0.75), bone_(NULL), qAdjust_(0, 0, 0, 1)
{
  txAd = new Gtk::Adjustment(0, -1000, 1000, 0.2, 1.0, 0.0);
  tyAd = new Gtk::Adjustment(0, -1000, 1000, 0.2, 1.0, 0.0);
  tzAd = new Gtk::Adjustment(0, -1000, 1000, 0.2, 1.0, 0.0);
  sxAd = new Gtk::Adjustment(1, -1000, 1000, 0.1, 1.0, 0.0);
  syAd = new Gtk::Adjustment(1, -1000, 1000, 0.1, 1.0, 0.0);
  szAd = new Gtk::Adjustment(1, -1000, 1000, 0.1, 1.0, 0.0);
}


StaticObject::StaticObject(CalBone *bone) : ball(CalVector(0, 0, 0), 1)
{
  StaticObject();
  bone_ = bone;
}

StaticObject::~StaticObject() {
  if(dialog) {
    delete dialog;
  }
  delete txAd;
  delete tyAd;
  delete tzAd;
	
  delete sxAd;
  delete syAd;
  delete szAd;
}

void StaticObject::attach(CalBone *bone) {
  bone_ = bone;
  //	ball.setPosition(bone->getTranslationAbsolute());
}

void StaticObject::detach() {
  bone_ = NULL;
  ball.setPosition(CalVector(0, 0, 0));
}

void StaticObject::setTranslation(double x, double y, double z) {
  txAd->set_value(x);
  tyAd->set_value(y);
  tzAd->set_value(z);
}

void StaticObject::setScale(double x) {
  sxAd->set_value(x);
  syAd->set_value(x);
  szAd->set_value(x);
}

// Kai Blin  2004-08-11
void StaticObject::setScale(double x, double y, double z) {
  sxAd->set_value(x);
  syAd->set_value(y);
  szAd->set_value(z);
}

void StaticObject::applyTransformation() {
  if(bone_) {
    CalVector transf = bone_->getTranslationAbsolute();
    glTranslatef(transf.x, transf.y, transf.z);
    CalQuaternion orient = bone_->getRotationBoneSpace();
    applyQuaternion(orient);
    orient *= qAdjust_;
    orient.conjugate();
    ball.adjustRotation(orient);
  } else {
    ball.adjustRotation(CalQuaternion(0, 0, 0, 0));
  }
  glTranslatef(tx(), ty(), tz());
  ball.update();
  applyCalMatrix(ball.value());
  glScalef(sx(), sy(), sz());
}

void StaticObject::adjustRotation(CalQuaternion q) {
  qAdjust_ = q;
}

void StaticObject::render() {
  glPushMatrix();
  glPushAttrib( GL_ENABLE_BIT );
  glDisable( GL_LIGHTING);
  applyTransformation();
  renderPivot();
  glPointSize(10.0f);
  glColor3f(0.2f, 0.8f, 0.2f);
  glBegin(GL_POINTS);
  glVertex3f(0, 0, 0);
  glEnd();
  /*	glLineWidth(3.0f);
	glColor3f(0.9f, 0.8f, 0.0f);
	glBegin(GL_LINES);
        glVertex3f(0, 0, 0);
        glVertex3f(0, -20, 0);
	glEnd();
	glPopAttrib();
  */	glPopMatrix();
}

void StaticObject::renderPivot() {
  glPushAttrib( GL_ENABLE_BIT );
  glDisable( GL_LIGHTING);
  glLineWidth(2.0f);
  glBegin(GL_LINES);
  glColor4f(1, 0, 0, 0.5);
  glVertex3f(0, 0, 0);
  glVertex3f(20, 0, 0);
  glColor4f(0, 1, 0, 0.5);
  glVertex3f(0, 0, 0);
  glVertex3f(0, 20, 0);
  glColor4f(0, 0, 1, 0.5);
  glVertex3f(0, 0, 0);
  glVertex3f(0, 0, 20);
  glEnd();
  glPopAttrib();
}

void StaticObject::createSetupDialog(Gtk::Window *main) {
  dialog = new Gtk::Dialog("Static object settings", *main, false);
  dialog->get_action_area()->set_layout(Gtk::BUTTONBOX_SPREAD);
  dialog->property_window_position().set_value(Gtk::WIN_POS_CENTER_ON_PARENT);
  dialog->set_transient_for(*main);
  Gtk::Button *pButton = dialog->add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  pButton->signal_clicked().connect(
                                    sigc::mem_fun(*dialog, &Gtk::Dialog::hide));
  pButton = dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  pButton->signal_clicked().connect(
                                    sigc::mem_fun(*dialog, &Gtk::Dialog::hide));
  dialog->get_vbox()->pack_start(*Gtk::manage(createControlsTable()));
  dialog->get_vbox()->show_all();
}


Gtk::Table *StaticObject::createControlsTable() {
  Gtk::Table *pTab = new Gtk::Table(4, 4);
  // Translation
  pTab->attach(*Gtk::manage(new Gtk::Label("Translation: ")),
               0, 1, 0, 1, Gtk::FILL, Gtk::FILL, 4, 4);
  Gtk::SpinButton *pSpin = new Gtk::SpinButton(*txAd, 1, 1);
  pTab->attach(*Gtk::manage(pSpin),
               1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
  pSpin = new Gtk::SpinButton(*tyAd, 1, 1);
  pTab->attach(*Gtk::manage(pSpin),
               2, 3, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
  pSpin = new Gtk::SpinButton(*tzAd, 1, 1);
  pTab->attach(*Gtk::manage(pSpin),
               3, 4, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);

  // Scale
  pTab->attach(*Gtk::manage(new Gtk::Label("Scale: ")),
               0, 1, 2, 3, Gtk::FILL, Gtk::FILL, 4, 4);
  pSpin = new Gtk::SpinButton(*sxAd, 1, 1);
  pTab->attach(*Gtk::manage(pSpin),
               1, 2, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
  pSpin = new Gtk::SpinButton(*syAd, 1, 1);
  pTab->attach(*Gtk::manage(pSpin),
               2, 3, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
  pSpin = new Gtk::SpinButton(*szAd, 1, 1);
  pTab->attach(*Gtk::manage(pSpin),
               3, 4, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
  return pTab;
}

void StaticObject::setup() {
  if(dialog) {
    dialog->run();
  }
}
